/**
 * Bundle of @devexpress/dx-react-grid-bootstrap3
 * Generated: 2017-10-25
 * Version: 1.0.0-alpha.14
 * License: https://js.devexpress.com/Licensing
 */

import React from 'react';
import PropTypes from 'prop-types';
import { ColumnChooser, DragDropContext, Grid, GroupPanelLayout, GroupingPanel, PagingPanel, TableColumnVisibility, TableEditColumn, TableEditRow, TableFilterRow, TableGroupRow, TableHeaderRow, TableLayout, TableRowDetail, TableSelection, TableView } from '@devexpress/dx-react-grid';
import { Pager, Pagination } from 'react-bootstrap';
import { firstRowOnPage, getTableRowColumnsWithColSpan, lastRowOnPage } from '@devexpress/dx-grid-core';
import { DragSource, Draggable, combineTemplates } from '@devexpress/dx-react-core';

var Root = function Root(_ref) {
  var headerTemplate = _ref.headerTemplate,
      bodyTemplate = _ref.bodyTemplate,
      footerTemplate = _ref.footerTemplate;
  return React.createElement(
    'div',
    { className: 'panel panel-default' },
    headerTemplate(),
    bodyTemplate(),
    footerTemplate()
  );
};

Root.propTypes = {
  headerTemplate: PropTypes.func.isRequired,
  bodyTemplate: PropTypes.func.isRequired,
  footerTemplate: PropTypes.func.isRequired
};

var Header = function Header(_ref2) {
  var children = _ref2.children;
  return children && React.createElement(
    'div',
    { className: 'panel-heading', style: { paddingBottom: '5px' } },
    children
  );
};

Header.propTypes = {
  children: PropTypes.node
};

Header.defaultProps = {
  children: null
};

var Footer = function Footer(_ref3) {
  var children = _ref3.children;
  return children && React.createElement(
    'div',
    { className: 'panel-footer' },
    children
  );
};

Footer.propTypes = {
  children: PropTypes.node
};

Footer.defaultProps = {
  children: null
};

var asyncGenerator = function () {
  function AwaitValue(value) {
    this.value = value;
  }

  function AsyncGenerator(gen) {
    var front, back;

    function send(key, arg) {
      return new Promise(function (resolve, reject) {
        var request = {
          key: key,
          arg: arg,
          resolve: resolve,
          reject: reject,
          next: null
        };

        if (back) {
          back = back.next = request;
        } else {
          front = back = request;
          resume(key, arg);
        }
      });
    }

    function resume(key, arg) {
      try {
        var result = gen[key](arg);
        var value = result.value;

        if (value instanceof AwaitValue) {
          Promise.resolve(value.value).then(function (arg) {
            resume("next", arg);
          }, function (arg) {
            resume("throw", arg);
          });
        } else {
          settle(result.done ? "return" : "normal", result.value);
        }
      } catch (err) {
        settle("throw", err);
      }
    }

    function settle(type, value) {
      switch (type) {
        case "return":
          front.resolve({
            value: value,
            done: true
          });
          break;

        case "throw":
          front.reject(value);
          break;

        default:
          front.resolve({
            value: value,
            done: false
          });
          break;
      }

      front = front.next;

      if (front) {
        resume(front.key, front.arg);
      } else {
        back = null;
      }
    }

    this._invoke = send;

    if (typeof gen.return !== "function") {
      this.return = undefined;
    }
  }

  if (typeof Symbol === "function" && Symbol.asyncIterator) {
    AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
      return this;
    };
  }

  AsyncGenerator.prototype.next = function (arg) {
    return this._invoke("next", arg);
  };

  AsyncGenerator.prototype.throw = function (arg) {
    return this._invoke("throw", arg);
  };

  AsyncGenerator.prototype.return = function (arg) {
    return this._invoke("return", arg);
  };

  return {
    wrap: function (fn) {
      return function () {
        return new AsyncGenerator(fn.apply(this, arguments));
      };
    },
    await: function (value) {
      return new AwaitValue(value);
    }
  };
}();





var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();





var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};



var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};









var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var Grid$1 = function Grid$$1(_ref) {
  var children = _ref.children,
      props = objectWithoutProperties(_ref, ['children']);
  return React.createElement(
    Grid,
    _extends({
      rootTemplate: Root,
      headerPlaceholderTemplate: Header,
      footerPlaceholderTemplate: Footer
    }, props),
    children
  );
};

Grid$1.propTypes = {
  children: PropTypes.oneOfType([PropTypes.arrayOf(PropTypes.node), PropTypes.node]).isRequired
};

var ColumnChooserContainer = function ColumnChooserContainer(_ref) {
  var children = _ref.children;
  return React.createElement(
    'div',
    { className: 'list-group' },
    children
  );
};

ColumnChooserContainer.propTypes = {
  children: PropTypes.arrayOf(PropTypes.node).isRequired
};

var ColumnChooserItem = function ColumnChooserItem(_ref) {
  var _ref$item = _ref.item,
      column = _ref$item.column,
      hidden = _ref$item.hidden,
      onToggle = _ref.onToggle;

  var handleChange = function handleChange() {
    return onToggle();
  };
  return React.createElement(
    'button',
    {
      className: 'list-group-item',
      style: { outline: 'none' },
      type: 'button',
      onClick: handleChange
    },
    React.createElement('input', {
      type: 'checkbox',
      tabIndex: -1,
      checked: !hidden,
      onChange: handleChange
    }),
    '\xA0',
    column.title || column.name
  );
};

ColumnChooserItem.propTypes = {
  item: PropTypes.shape({
    column: PropTypes.shape({
      name: PropTypes.string
    }),
    hidden: PropTypes.bool
  }).isRequired,
  onToggle: PropTypes.func
};

ColumnChooserItem.defaultProps = {
  onToggle: function onToggle() {}
};

var containerTemplate = function containerTemplate(props) {
  return React.createElement(ColumnChooserContainer, props);
};
var itemTemplate = function itemTemplate(props) {
  return React.createElement(ColumnChooserItem, props);
};

var ColumnChooser$1 = function ColumnChooser$$1(props) {
  return React.createElement(ColumnChooser, _extends({
    containerTemplate: containerTemplate,
    itemTemplate: itemTemplate
  }, props));
};

var Container = function Container(_ref) {
  var clientOffset = _ref.clientOffset,
      columns = _ref.columns,
      columnTemplate = _ref.columnTemplate;
  return React.createElement(
    'ul',
    {
      className: 'list-group',
      style: {
        cursor: 'move',
        position: 'fixed',
        zIndex: 1000,
        left: 0,
        top: 0,
        display: 'inline-block',
        transform: 'translate(calc(' + clientOffset.x + 'px - 50%), calc(' + clientOffset.y + 'px - 50%))'
      }
    },
    columns.map(function (column) {
      return React.cloneElement(columnTemplate({ column: column }), { key: column.name });
    })
  );
};

Container.propTypes = {
  clientOffset: PropTypes.shape({
    x: PropTypes.number.isRequired,
    y: PropTypes.number.isRequired
  }).isRequired,
  columns: PropTypes.array.isRequired,
  columnTemplate: PropTypes.func.isRequired
};

var Column = function Column(_ref2) {
  var column = _ref2.column;
  return React.createElement(
    'li',
    { className: 'list-group-item' },
    column.title
  );
};

Column.propTypes = {
  column: PropTypes.object.isRequired
};

var containerTemplate$1 = function containerTemplate(props) {
  return React.createElement(Container, props);
};
var columnTemplate = function columnTemplate(props) {
  return React.createElement(Column, props);
};

var DragDropContext$1 = function (_React$PureComponent) {
  inherits(DragDropContext$$1, _React$PureComponent);

  function DragDropContext$$1() {
    classCallCheck(this, DragDropContext$$1);
    return possibleConstructorReturn(this, (DragDropContext$$1.__proto__ || Object.getPrototypeOf(DragDropContext$$1)).apply(this, arguments));
  }

  createClass(DragDropContext$$1, [{
    key: 'render',
    value: function render() {
      return React.createElement(DragDropContext, _extends({
        containerTemplate: containerTemplate$1,
        columnTemplate: columnTemplate
      }, this.props));
    }
  }]);
  return DragDropContext$$1;
}(React.PureComponent);

var PageSizeSelector = function PageSizeSelector(_ref) {
  var pageSize = _ref.pageSize,
      onPageSizeChange = _ref.onPageSizeChange,
      allowedPageSizes = _ref.allowedPageSizes,
      showAllText = _ref.showAllText;
  return React.createElement(
    'div',
    { style: { display: 'inline-block' } },
    React.createElement(
      'select',
      {
        className: 'form-control visible-xs-inline-block',
        style: { width: 'auto' },
        value: pageSize,
        onChange: function onChange(e) {
          return onPageSizeChange(parseInt(e.target.value, 10));
        }
      },
      allowedPageSizes.map(function (val) {
        return React.createElement(
          'option',
          { key: val, value: val },
          val || showAllText
        );
      })
    ),
    React.createElement(
      'ul',
      {
        className: 'pagination hidden-xs',
        style: {
          margin: 0,
          verticalAlign: 'bottom'
        }
      },
      allowedPageSizes.map(function (item) {
        return React.createElement(
          'li',
          { key: item, className: item === pageSize ? 'active' : '' },
          React.createElement(
            'a',
            {
              href: '#',
              onClick: function onClick(e) {
                e.preventDefault();
                onPageSizeChange(item);
              }
            },
            item || showAllText
          )
        );
      })
    )
  );
};

PageSizeSelector.propTypes = {
  pageSize: PropTypes.number.isRequired,
  onPageSizeChange: PropTypes.func.isRequired,
  allowedPageSizes: PropTypes.arrayOf(PropTypes.number).isRequired,
  showAllText: PropTypes.string
};

PageSizeSelector.defaultProps = {
  showAllText: 'All'
};

var Pager$1 = function Pager$$1(_ref) {
  var currentPage = _ref.currentPage,
      onCurrentPageChange = _ref.onCurrentPageChange,
      totalPages = _ref.totalPages,
      pageSize = _ref.pageSize,
      onPageSizeChange = _ref.onPageSizeChange,
      allowedPageSizes = _ref.allowedPageSizes,
      totalCount = _ref.totalCount,
      showAllText = _ref.showAllText;

  var firstRow = firstRowOnPage(currentPage, pageSize, totalCount);
  var lastRow = lastRowOnPage(currentPage, pageSize, totalCount);

  return React.createElement(
    'div',
    { className: 'clearfix' },
    !!allowedPageSizes.length && React.createElement(PageSizeSelector, {
      pageSize: pageSize,
      onPageSizeChange: onPageSizeChange,
      allowedPageSizes: allowedPageSizes,
      showAllText: showAllText
    }),
    React.createElement(Pagination, {
      style: {
        margin: 0,
        verticalAlign: 'bottom'
      },
      className: 'pull-right hidden-xs',
      items: totalPages || 1,
      activePage: currentPage + 1,
      boundaryLinks: true,
      maxButtons: 3,
      onSelect: function onSelect(page) {
        return onCurrentPageChange(page - 1);
      }
    }),
    React.createElement(
      Pager,
      {
        className: 'pull-right visible-xs',
        style: { margin: 0 }
      },
      React.createElement(
        Pager.Item,
        {
          disabled: currentPage === 0,
          onClick: function onClick() {
            return onCurrentPageChange(currentPage - 1);
          }
        },
        '\xAB'
      ),
      ' ',
      React.createElement(
        Pager.Item,
        {
          disabled: currentPage === totalPages - 1 || totalCount === 0,
          onClick: function onClick() {
            return onCurrentPageChange(currentPage + 1);
          }
        },
        '\xBB'
      )
    ),
    React.createElement(
      'span',
      { className: 'pull-right visible-xs', style: { marginRight: '20px' } },
      React.createElement(
        'span',
        { style: { display: 'inline-block', verticalAlign: 'middle', lineHeight: '32px' } },
        '' + firstRow + (firstRow < lastRow ? '-' + lastRow : '') + ' of ' + totalCount
      )
    )
  );
};

Pager$1.propTypes = {
  currentPage: PropTypes.number.isRequired,
  onCurrentPageChange: PropTypes.func.isRequired,
  totalPages: PropTypes.number.isRequired,
  pageSize: PropTypes.number.isRequired,
  onPageSizeChange: PropTypes.func.isRequired,
  allowedPageSizes: PropTypes.arrayOf(PropTypes.number).isRequired,
  totalCount: PropTypes.number.isRequired,
  showAllText: PropTypes.string
};

Pager$1.defaultProps = {
  showAllText: undefined
};

var PagingPanel$1 = function (_React$PureComponent) {
  inherits(PagingPanel$$1, _React$PureComponent);

  function PagingPanel$$1() {
    classCallCheck(this, PagingPanel$$1);
    return possibleConstructorReturn(this, (PagingPanel$$1.__proto__ || Object.getPrototypeOf(PagingPanel$$1)).apply(this, arguments));
  }

  createClass(PagingPanel$$1, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          showAllText = _props.showAllText,
          restProps = objectWithoutProperties(_props, ['showAllText']);


      return React.createElement(PagingPanel, _extends({
        pagerTemplate: function pagerTemplate(props) {
          return React.createElement(Pager$1, _extends({
            showAllText: showAllText
          }, props));
        }
      }, restProps));
    }
  }]);
  return PagingPanel$$1;
}(React.PureComponent);

PagingPanel$1.propTypes = {
  showAllText: PropTypes.string
};
PagingPanel$1.defaultProps = {
  showAllText: undefined
};

var defaultTextStyle = {
  padding: '7px 0',
  marginBottom: '5px',
  display: 'inline-block'
};

var getDefaultText = function getDefaultText(allowDragging, allowUngroupingByClick) {
  if (allowDragging) {
    return React.createElement(
      'span',
      { style: defaultTextStyle },
      'Drag a column header here to group by that column'
    );
  }
  if (allowUngroupingByClick) {
    return React.createElement(
      'span',
      { style: defaultTextStyle },
      'Click \xA0',
      React.createElement('i', {
        className: 'glyphicon glyphicon-th-list',
        style: {
          top: '0',
          fontSize: '9px'
        }
      }),
      '\xA0 icon in the column header to group by that column'
    );
  }
  return React.createElement(
    'span',
    { style: defaultTextStyle },
    'Grouping is not available'
  );
};

var PanelTemplate = function PanelTemplate(_ref) {
  var items = _ref.items;
  return React.createElement(
    'div',
    null,
    items
  );
};

PanelTemplate.propTypes = {
  items: PropTypes.arrayOf(PropTypes.node).isRequired
};

var panelTemplate = function panelTemplate(props) {
  return React.createElement(PanelTemplate, props);
};

var GroupPanel = function GroupPanel(_ref2) {
  var groupByColumnText = _ref2.groupByColumnText,
      restProps = objectWithoutProperties(_ref2, ['groupByColumnText']);
  return React.createElement(GroupPanelLayout, _extends({
    groupByColumnText: groupByColumnText || getDefaultText(restProps.allowDragging, restProps.allowUngroupingByClick),
    panelTemplate: panelTemplate
  }, restProps));
};

GroupPanel.propTypes = {
  groupByColumnText: PropTypes.string
};

GroupPanel.defaultProps = {
  groupByColumnText: undefined
};

var SortingIndicator = function SortingIndicator(_ref) {
  var direction = _ref.direction,
      style = _ref.style;
  return React.createElement('i', {
    className: 'glyphicon glyphicon-arrow-' + (direction === 'desc' ? 'down' : 'up'),
    style: _extends({
      top: '0',
      fontSize: '9px'
    }, style)
  });
};

SortingIndicator.propTypes = {
  direction: PropTypes.oneOf(['asc', 'desc']),
  style: PropTypes.object
};

SortingIndicator.defaultProps = {
  direction: null,
  style: null
};

var GroupPanelItem = function GroupPanelItem(_ref) {
  var column = _ref.column,
      draft = _ref.draft,
      groupByColumn = _ref.groupByColumn,
      allowUngroupingByClick = _ref.allowUngroupingByClick,
      allowSorting = _ref.allowSorting,
      sortingDirection = _ref.sortingDirection,
      changeSortingDirection = _ref.changeSortingDirection;
  return React.createElement(
    'div',
    {
      className: 'btn btn-default',
      style: _extends({
        marginRight: '5px',
        marginBottom: '5px'
      }, draft ? { opacity: 0.3 } : null)
    },
    React.createElement(
      'span',
      {
        onClick: function onClick(e) {
          if (!allowSorting) return;
          var cancelSortingRelatedKey = e.metaKey || e.ctrlKey;
          changeSortingDirection({
            keepOther: e.shiftKey || cancelSortingRelatedKey,
            cancel: cancelSortingRelatedKey,
            columnName: column.name
          });
        }
      },
      column.title || column.name,
      allowSorting && sortingDirection && React.createElement(
        'span',
        null,
        '\xA0',
        React.createElement(SortingIndicator, {
          direction: sortingDirection
        })
      )
    ),
    allowUngroupingByClick && React.createElement(
      'span',
      null,
      '\xA0',
      React.createElement('i', {
        className: 'glyphicon glyphicon-remove',
        style: {
          top: '0',
          fontSize: '9px',
          margin: '-5px',
          padding: '5px'
        },
        onClick: function onClick() {
          return groupByColumn({ columnName: column.name });
        }
      })
    )
  );
};

GroupPanelItem.propTypes = {
  column: PropTypes.shape({
    title: PropTypes.string
  }).isRequired,
  draft: PropTypes.bool,
  allowSorting: PropTypes.bool,
  sortingDirection: PropTypes.oneOf(['asc', 'desc', null]),
  changeSortingDirection: PropTypes.func,
  groupByColumn: PropTypes.func,
  allowUngroupingByClick: PropTypes.bool
};

GroupPanelItem.defaultProps = {
  draft: false,
  allowSorting: false,
  sortingDirection: undefined,
  changeSortingDirection: undefined,
  groupByColumn: undefined,
  allowUngroupingByClick: false
};

var defaultCellTemplate = function defaultCellTemplate(props) {
  return React.createElement(GroupPanelItem, props);
};

var GroupingPanel$1 = function (_React$PureComponent) {
  inherits(GroupingPanel$$1, _React$PureComponent);

  function GroupingPanel$$1() {
    classCallCheck(this, GroupingPanel$$1);
    return possibleConstructorReturn(this, (GroupingPanel$$1.__proto__ || Object.getPrototypeOf(GroupingPanel$$1)).apply(this, arguments));
  }

  createClass(GroupingPanel$$1, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          groupByColumnText = _props.groupByColumnText,
          groupPanelItemTemplate = _props.groupPanelItemTemplate,
          restProps = objectWithoutProperties(_props, ['groupByColumnText', 'groupPanelItemTemplate']);


      return React.createElement(GroupingPanel, _extends({
        groupPanelTemplate: function groupPanelTemplate(props) {
          return React.createElement(GroupPanel, _extends({
            groupByColumnText: groupByColumnText,
            groupPanelItemTemplate: combineTemplates(groupPanelItemTemplate, defaultCellTemplate)
          }, props));
        }
      }, restProps));
    }
  }]);
  return GroupingPanel$$1;
}(React.PureComponent);

GroupingPanel$1.propTypes = {
  allowSorting: PropTypes.bool,
  groupByColumnText: PropTypes.string,
  groupPanelItemTemplate: PropTypes.func
};

GroupingPanel$1.defaultProps = {
  allowSorting: false,
  groupByColumnText: undefined,
  groupPanelItemTemplate: undefined
};

var TableDetailToggleCell = function TableDetailToggleCell(_ref) {
  var style = _ref.style,
      expanded = _ref.expanded,
      toggleExpanded = _ref.toggleExpanded;
  return React.createElement(
    'td',
    {
      style: _extends({
        cursor: 'pointer'
      }, style),
      onClick: function onClick(e) {
        e.stopPropagation();
        toggleExpanded();
      }
    },
    React.createElement('i', {
      className: 'glyphicon glyphicon-triangle-' + (expanded ? 'bottom' : 'right'),
      style: {
        fontSize: '9px',
        top: '0'
      }
    })
  );
};

TableDetailToggleCell.propTypes = {
  style: PropTypes.object,
  expanded: PropTypes.bool,
  toggleExpanded: PropTypes.func
};

TableDetailToggleCell.defaultProps = {
  style: null,
  expanded: false,
  toggleExpanded: function toggleExpanded() {}
};

var TableDetailCell = function TableDetailCell(_ref) {
  var colSpan = _ref.colSpan,
      style = _ref.style,
      template = _ref.template;
  return React.createElement(
    'td',
    { style: style, colSpan: colSpan, className: 'active' },
    template()
  );
};

TableDetailCell.propTypes = {
  style: PropTypes.object,
  colSpan: PropTypes.number,
  template: PropTypes.func.isRequired
};

TableDetailCell.defaultProps = {
  style: null,
  colSpan: 1
};

var TableRow = function TableRow(_ref) {
  var children = _ref.children,
      style = _ref.style;
  return React.createElement(
    'tr',
    {
      style: style
    },
    children
  );
};

TableRow.propTypes = {
  children: PropTypes.node,
  style: PropTypes.object
};

TableRow.defaultProps = {
  children: null,
  style: null
};

var defaultDetailToggleCellTemplate = function defaultDetailToggleCellTemplate(props) {
  return React.createElement(TableDetailToggleCell, props);
};
var defaultDetailCellTemplate = function defaultDetailCellTemplate(props) {
  return React.createElement(TableDetailCell, props);
};
var defaultDetailRowTemplate = function defaultDetailRowTemplate(props) {
  return React.createElement(TableRow, props);
};

var TableRowDetail$1 = function (_React$PureComponent) {
  inherits(TableRowDetail$$1, _React$PureComponent);

  function TableRowDetail$$1() {
    classCallCheck(this, TableRowDetail$$1);
    return possibleConstructorReturn(this, (TableRowDetail$$1.__proto__ || Object.getPrototypeOf(TableRowDetail$$1)).apply(this, arguments));
  }

  createClass(TableRowDetail$$1, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          detailCellTemplate = _props.detailCellTemplate,
          detailRowTemplate = _props.detailRowTemplate,
          detailToggleCellTemplate = _props.detailToggleCellTemplate,
          restProps = objectWithoutProperties(_props, ['detailCellTemplate', 'detailRowTemplate', 'detailToggleCellTemplate']);


      return React.createElement(TableRowDetail, _extends({
        detailToggleCellTemplate: combineTemplates(detailToggleCellTemplate, defaultDetailToggleCellTemplate),
        detailCellTemplate: combineTemplates(detailCellTemplate, defaultDetailCellTemplate),
        detailRowTemplate: combineTemplates(detailRowTemplate, defaultDetailRowTemplate),
        detailToggleCellWidth: 25
      }, restProps));
    }
  }]);
  return TableRowDetail$$1;
}(React.PureComponent);

TableRowDetail$1.propTypes = {
  detailRowTemplate: PropTypes.func,
  detailCellTemplate: PropTypes.func,
  detailToggleCellTemplate: PropTypes.func
};

TableRowDetail$1.defaultProps = {
  detailRowTemplate: undefined,
  detailCellTemplate: undefined,
  detailToggleCellTemplate: undefined
};

var TableGroupCell = function TableGroupCell(_ref) {
  var style = _ref.style,
      colSpan = _ref.colSpan,
      row = _ref.row,
      column = _ref.column,
      isExpanded = _ref.isExpanded,
      toggleGroupExpanded = _ref.toggleGroupExpanded,
      children = _ref.children;
  return React.createElement(
    'td',
    {
      colSpan: colSpan,
      style: _extends({
        cursor: 'pointer'
      }, style),
      onClick: toggleGroupExpanded
    },
    React.createElement('i', {
      className: 'glyphicon glyphicon-triangle-' + (isExpanded ? 'bottom' : 'right'),
      style: {
        fontSize: '9px',
        top: 0,
        marginRight: '10px'
      }
    }),
    React.createElement(
      'strong',
      null,
      column.title || column.name,
      ': '
    ),
    children || row.value
  );
};

TableGroupCell.propTypes = {
  style: PropTypes.object,
  colSpan: PropTypes.number,
  row: PropTypes.object,
  column: PropTypes.object,
  isExpanded: PropTypes.bool,
  toggleGroupExpanded: PropTypes.func,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)])
};

TableGroupCell.defaultProps = {
  style: null,
  colSpan: 1,
  row: {},
  column: {},
  isExpanded: false,
  toggleGroupExpanded: function toggleGroupExpanded() {},
  children: undefined
};

var defaultGroupCellTemplate = function defaultGroupCellTemplate(props) {
  return React.createElement(TableGroupCell, props);
};
var defaultGroupRowTemplate = function defaultGroupRowTemplate(props) {
  return React.createElement(TableRow, props);
};

var TableGroupRow$1 = function (_React$PureComponent) {
  inherits(TableGroupRow$$1, _React$PureComponent);

  function TableGroupRow$$1() {
    classCallCheck(this, TableGroupRow$$1);
    return possibleConstructorReturn(this, (TableGroupRow$$1.__proto__ || Object.getPrototypeOf(TableGroupRow$$1)).apply(this, arguments));
  }

  createClass(TableGroupRow$$1, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          groupCellTemplate = _props.groupCellTemplate,
          groupRowTemplate = _props.groupRowTemplate,
          restProps = objectWithoutProperties(_props, ['groupCellTemplate', 'groupRowTemplate']);


      return React.createElement(TableGroupRow, _extends({
        groupCellTemplate: combineTemplates(groupCellTemplate, defaultGroupCellTemplate),
        groupRowTemplate: combineTemplates(groupRowTemplate, defaultGroupRowTemplate),
        groupIndentColumnWidth: 20
      }, restProps));
    }
  }]);
  return TableGroupRow$$1;
}(React.PureComponent);

TableGroupRow$1.propTypes = {
  groupRowTemplate: PropTypes.func,
  groupCellTemplate: PropTypes.func
};

TableGroupRow$1.defaultProps = {
  groupRowTemplate: undefined,
  groupCellTemplate: undefined
};

var TableSelectAllCell = function TableSelectAllCell(_ref) {
  var style = _ref.style,
      allSelected = _ref.allSelected,
      someSelected = _ref.someSelected,
      selectionAvailable = _ref.selectionAvailable,
      toggleAll = _ref.toggleAll;
  return React.createElement(
    'th',
    {
      style: _extends({
        cursor: selectionAvailable && 'pointer'
      }, style),
      onClick: function onClick(e) {
        if (!selectionAvailable) return;

        e.stopPropagation();
        toggleAll();
      }
    },
    React.createElement('input', {
      style: {
        cursor: selectionAvailable && 'pointer',
        margin: 0
      },
      type: 'checkbox',
      disabled: !selectionAvailable,
      checked: allSelected,
      ref: function ref(_ref2) {
        if (_ref2) {
          var checkbox = _ref2;
          checkbox.indeterminate = someSelected;
        }
      },
      onChange: toggleAll,
      onClick: function onClick(e) {
        return e.stopPropagation();
      }
    })
  );
};

TableSelectAllCell.propTypes = {
  style: PropTypes.object,
  allSelected: PropTypes.bool,
  someSelected: PropTypes.bool,
  selectionAvailable: PropTypes.bool,
  toggleAll: PropTypes.func
};

TableSelectAllCell.defaultProps = {
  style: null,
  allSelected: false,
  someSelected: false,
  selectionAvailable: false,
  toggleAll: function toggleAll() {}
};

var TableSelectCell = function TableSelectCell(_ref) {
  var style = _ref.style,
      selected = _ref.selected,
      changeSelected = _ref.changeSelected;
  return React.createElement(
    'td',
    {
      style: _extends({
        cursor: 'pointer'
      }, style),
      onClick: function onClick(e) {
        e.stopPropagation();
        changeSelected();
      }
    },
    React.createElement('input', {
      style: {
        cursor: 'pointer',
        margin: 0
      },
      type: 'checkbox',
      checked: selected,
      onChange: changeSelected,
      onClick: function onClick(e) {
        return e.stopPropagation();
      }
    })
  );
};
TableSelectCell.defaultProps = {
  style: null,
  selected: false,
  changeSelected: function changeSelected() {}
};
TableSelectCell.propTypes = {
  style: PropTypes.object,
  selected: PropTypes.bool,
  changeSelected: PropTypes.func
};

var TableSelectRow = function TableSelectRow(_ref) {
  var selected = _ref.selected,
      children = _ref.children,
      style = _ref.style,
      changeSelected = _ref.changeSelected,
      selectByRowClick = _ref.selectByRowClick;
  return React.createElement(
    'tr',
    {
      style: style,
      className: selected ? 'active' : '',
      onClick: function onClick(e) {
        if (!selectByRowClick) return;
        e.stopPropagation();
        changeSelected();
      }
    },
    children
  );
};

TableSelectRow.propTypes = {
  selected: PropTypes.bool,
  children: PropTypes.node,
  changeSelected: PropTypes.func,
  selectByRowClick: PropTypes.bool,
  style: PropTypes.object
};

TableSelectRow.defaultProps = {
  children: null,
  changeSelected: function changeSelected() {},
  selected: false,
  selectByRowClick: false,
  style: null
};

var selectCellTemplate = function selectCellTemplate(props) {
  return React.createElement(TableSelectCell, props);
};
var selectAllCellTemplate = function selectAllCellTemplate(props) {
  return React.createElement(TableSelectAllCell, props);
};
var selectRowTemplate = function selectRowTemplate(props) {
  return React.createElement(TableSelectRow, props);
};

var TableSelection$1 = function (_React$PureComponent) {
  inherits(TableSelection$$1, _React$PureComponent);

  function TableSelection$$1() {
    classCallCheck(this, TableSelection$$1);
    return possibleConstructorReturn(this, (TableSelection$$1.__proto__ || Object.getPrototypeOf(TableSelection$$1)).apply(this, arguments));
  }

  createClass(TableSelection$$1, [{
    key: 'render',
    value: function render() {
      return React.createElement(TableSelection, _extends({
        selectCellTemplate: selectCellTemplate,
        selectRowTemplate: selectRowTemplate,
        selectAllCellTemplate: selectAllCellTemplate,
        selectionColumnWidth: 40
      }, this.props));
    }
  }]);
  return TableSelection$$1;
}(React.PureComponent);

var MINIMAL_COLUMN_WIDTH = 120;

/* eslint-disable react/prop-types */
var tableTemplate = function tableTemplate(_ref) {
  var children = _ref.children,
      restProps = objectWithoutProperties(_ref, ['children']);
  return React.createElement(
    'table',
    _extends({
      className: 'table'
    }, restProps, {
      style: _extends({}, restProps.style, {
        overflow: 'hidden'
      })
    }),
    children
  );
};
var headTemplate = function headTemplate(_ref2) {
  var children = _ref2.children,
      restProps = objectWithoutProperties(_ref2, ['children']);
  return React.createElement(
    'thead',
    restProps,
    children
  );
};
var bodyTemplate = function bodyTemplate(_ref3) {
  var children = _ref3.children,
      restProps = objectWithoutProperties(_ref3, ['children']);
  return React.createElement(
    'tbody',
    restProps,
    children
  );
};

var Table = function Table(_ref4) {
  var headerRows = _ref4.headerRows,
      bodyRows = _ref4.bodyRows,
      columns = _ref4.columns,
      cellTemplate = _ref4.cellTemplate,
      rowTemplate = _ref4.rowTemplate,
      allowColumnReordering = _ref4.allowColumnReordering,
      setColumnOrder = _ref4.setColumnOrder;
  return React.createElement(TableLayout, {
    className: 'table-responsive',
    headerRows: headerRows,
    rows: bodyRows,
    columns: columns,
    minColumnWidth: MINIMAL_COLUMN_WIDTH,
    tableTemplate: tableTemplate,
    headTemplate: headTemplate,
    bodyTemplate: bodyTemplate,
    rowTemplate: rowTemplate,
    cellTemplate: cellTemplate,
    allowColumnReordering: allowColumnReordering,
    setColumnOrder: setColumnOrder
  });
};

Table.propTypes = {
  headerRows: PropTypes.array.isRequired,
  bodyRows: PropTypes.array.isRequired,
  columns: PropTypes.array.isRequired,
  cellTemplate: PropTypes.func.isRequired,
  rowTemplate: PropTypes.func.isRequired,
  allowColumnReordering: PropTypes.bool.isRequired,
  setColumnOrder: PropTypes.func
};

Table.defaultProps = {
  setColumnOrder: function setColumnOrder() {}
};

var TableCell = function TableCell(_ref) {
  var style = _ref.style,
      column = _ref.column,
      value = _ref.value,
      children = _ref.children;
  return React.createElement(
    'td',
    {
      style: _extends({
        whiteSpace: 'nowrap',
        overflow: 'hidden',
        textOverflow: 'ellipsis',
        textAlign: column.align || 'left'
      }, style)
    },
    children || value
  );
};

TableCell.propTypes = {
  style: PropTypes.object,
  value: PropTypes.any,
  column: PropTypes.object,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)])
};

TableCell.defaultProps = {
  style: null,
  value: undefined,
  column: {},
  children: undefined
};

var TableStubCell = function TableStubCell(_ref) {
  var style = _ref.style;
  return React.createElement('td', {
    style: _extends({
      padding: 0
    }, style)
  });
};

TableStubCell.propTypes = {
  style: PropTypes.object
};

TableStubCell.defaultProps = {
  style: null
};

var TableStubHeaderCell = function TableStubHeaderCell(_ref) {
  var style = _ref.style;
  return React.createElement('th', {
    style: _extends({
      padding: 0
    }, style)
  });
};

TableStubHeaderCell.propTypes = {
  style: PropTypes.object
};

TableStubHeaderCell.defaultProps = {
  style: null
};

var TableNoDataCell = function TableNoDataCell(_ref) {
  var style = _ref.style,
      colSpan = _ref.colSpan;
  return React.createElement(
    'td',
    {
      style: _extends({
        textAlign: 'center',
        padding: '40px 0'
      }, style),
      colSpan: colSpan
    },
    React.createElement(
      'big',
      { className: 'text-muted' },
      'No data'
    )
  );
};

TableNoDataCell.propTypes = {
  style: PropTypes.object,
  colSpan: PropTypes.number
};

TableNoDataCell.defaultProps = {
  style: null,
  colSpan: 1
};

var tableLayoutTemplate = function tableLayoutTemplate(props) {
  return React.createElement(Table, props);
};
var defaultRowTemplate = function defaultRowTemplate(props) {
  return React.createElement(TableRow, props);
};
var defaultNoDataRowTemplate = function defaultNoDataRowTemplate(props) {
  return React.createElement(TableRow, props);
};
var defaultCellTemplate$1 = function defaultCellTemplate(props) {
  return React.createElement(TableCell, props);
};
var defaultStubCellTemplate = function defaultStubCellTemplate(props) {
  return React.createElement(TableStubCell, props);
};
var defaultStubHeaderCellTemplate = function defaultStubHeaderCellTemplate(props) {
  return React.createElement(TableStubHeaderCell, props);
};
var defaultNoDataCellTemplate = function defaultNoDataCellTemplate(props) {
  return React.createElement(TableNoDataCell, props);
};

var TableView$1 = function (_React$PureComponent) {
  inherits(TableView$$1, _React$PureComponent);

  function TableView$$1() {
    classCallCheck(this, TableView$$1);
    return possibleConstructorReturn(this, (TableView$$1.__proto__ || Object.getPrototypeOf(TableView$$1)).apply(this, arguments));
  }

  createClass(TableView$$1, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          tableCellTemplate = _props.tableCellTemplate,
          tableRowTemplate = _props.tableRowTemplate,
          tableNoDataRowTemplate = _props.tableNoDataRowTemplate,
          tableStubCellTemplate = _props.tableStubCellTemplate,
          tableStubHeaderCellTemplate = _props.tableStubHeaderCellTemplate,
          tableNoDataCellTemplate = _props.tableNoDataCellTemplate,
          restProps = objectWithoutProperties(_props, ['tableCellTemplate', 'tableRowTemplate', 'tableNoDataRowTemplate', 'tableStubCellTemplate', 'tableStubHeaderCellTemplate', 'tableNoDataCellTemplate']);


      return React.createElement(TableView, _extends({
        tableLayoutTemplate: tableLayoutTemplate,
        tableRowTemplate: combineTemplates(tableRowTemplate, defaultRowTemplate),
        tableNoDataRowTemplate: combineTemplates(tableNoDataRowTemplate, defaultNoDataRowTemplate),
        tableCellTemplate: combineTemplates(tableCellTemplate, defaultCellTemplate$1),
        tableStubCellTemplate: combineTemplates(tableStubCellTemplate, defaultStubCellTemplate),
        tableStubHeaderCellTemplate: combineTemplates(tableStubHeaderCellTemplate, defaultStubHeaderCellTemplate),
        tableNoDataCellTemplate: combineTemplates(tableNoDataCellTemplate, defaultNoDataCellTemplate)
      }, restProps));
    }
  }]);
  return TableView$$1;
}(React.PureComponent);

TableView$1.propTypes = {
  tableCellTemplate: PropTypes.func,
  tableRowTemplate: PropTypes.func,
  tableNoDataRowTemplate: PropTypes.func,
  tableStubCellTemplate: PropTypes.func,
  tableStubHeaderCellTemplate: PropTypes.func,
  tableNoDataCellTemplate: PropTypes.func
};

TableView$1.defaultProps = {
  tableCellTemplate: undefined,
  tableRowTemplate: undefined,
  tableNoDataRowTemplate: undefined,
  tableStubCellTemplate: undefined,
  tableStubHeaderCellTemplate: undefined,
  tableNoDataCellTemplate: undefined
};

var Sizer = function (_React$Component) {
  inherits(Sizer, _React$Component);

  function Sizer() {
    classCallCheck(this, Sizer);
    return possibleConstructorReturn(this, (Sizer.__proto__ || Object.getPrototypeOf(Sizer)).apply(this, arguments));
  }

  createClass(Sizer, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      var watchSizes = function watchSizes() {
        var rect = _this2.root.getBoundingClientRect();
        _this2.sizeUpdated(rect.width, rect.height);
        // eslint-disable-next-line no-undef
        _this2.raf = requestAnimationFrame(watchSizes);
      };
      watchSizes();
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      // eslint-disable-next-line no-undef
      cancelAnimationFrame(this.raf);
    }
  }, {
    key: 'sizeUpdated',
    value: function sizeUpdated(newWidth, newHeight) {
      var _props = this.props,
          height = _props.height,
          onHeightChange = _props.onHeightChange,
          width = _props.width,
          onWidthChange = _props.onWidthChange;


      if (height !== undefined && newHeight !== height) {
        onHeightChange(newHeight);
      }
      if (width !== undefined && newWidth !== width) {
        onWidthChange(newWidth);
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _this3 = this;

      var _props2 = this.props,
          children = _props2.children,
          height = _props2.height,
          onHeightChange = _props2.onHeightChange,
          width = _props2.width,
          onWidthChange = _props2.onWidthChange,
          restProps = objectWithoutProperties(_props2, ['children', 'height', 'onHeightChange', 'width', 'onWidthChange']);


      return React.createElement(
        'div',
        _extends({ ref: function ref(_ref) {
            _this3.root = _ref;
          } }, restProps),
        children
      );
    }
  }]);
  return Sizer;
}(React.Component);
Sizer.defaultProps = {
  height: undefined,
  onHeightChange: function onHeightChange() {},
  width: undefined,
  onWidthChange: function onWidthChange() {}
};
Sizer.propTypes = {
  height: PropTypes.number,
  onHeightChange: PropTypes.func,
  width: PropTypes.number,
  onWidthChange: PropTypes.func,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)]).isRequired
};

var isSafari = void 0;

var WindowedScroller = function (_React$Component) {
  inherits(WindowedScroller, _React$Component);

  function WindowedScroller(props) {
    classCallCheck(this, WindowedScroller);

    var _this = possibleConstructorReturn(this, (WindowedScroller.__proto__ || Object.getPrototypeOf(WindowedScroller)).call(this, props));

    _this.state = {
      viewport: {
        top: 0, left: 0, width: 0, height: 0
      },
      offsetWidth: 0
    };

    _this.requestViewportUpdate = _this.requestViewportUpdate.bind(_this);
    return _this;
  }

  createClass(WindowedScroller, [{
    key: 'getChildContext',
    value: function getChildContext() {
      return {
        virtualHost: {
          viewport: this.state.viewport
        }
      };
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      if (isSafari === undefined) {
        // eslint-disable-next-line no-undef
        isSafari = /Safari/.test(navigator.userAgent) && /Apple Computer/.test(navigator.vendor);
      }
      setTimeout(function () {
        _this2.requestViewportUpdate();
      });
    }
  }, {
    key: 'requestViewportUpdate',
    value: function requestViewportUpdate() {
      if (isSafari) {
        // eslint-disable-next-line no-undef
        requestAnimationFrame(this.updataViewport.bind(this));
      } else {
        this.updataViewport();
      }
    }
  }, {
    key: 'updataViewport',
    value: function updataViewport() {
      if (!this.root) return;

      var oldViewport = this.state.viewport;
      var viewport = {
        top: this.root.scrollTop,
        left: this.root.scrollLeft,
        width: this.root.clientWidth,
        height: this.root.clientHeight
      };

      // Prevent iOS to flicker in bounces =(
      if (viewport.top < 0 || viewport.left < 0 || viewport.left + viewport.width > this.root.scrollWidth || viewport.top + viewport.height > this.root.scrollHeight) {
        return;
      }

      // Optimize performance
      if (oldViewport.top !== viewport.top || oldViewport.left !== viewport.left || oldViewport.width !== viewport.width || oldViewport.height !== viewport.height) {
        this.setState({ viewport: viewport });
        this.props.onViewportChange(viewport);
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _this3 = this;

      return React.createElement(
        Sizer,
        {
          style: { width: '100%', height: '100%' },
          width: this.state.offsetWidth,
          onWidthChange: function onWidthChange(width) {
            _this3.setState({ offsetWidth: width });
            _this3.requestViewportUpdate();
          }
        },
        React.createElement(
          'div',
          {
            ref: function ref(_ref) {
              _this3.root = _ref;
            },
            onScroll: this.requestViewportUpdate,
            style: {
              overflow: 'auto',
              width: '100%',
              height: '100%',
              WebkitOverflowScrolling: 'touch',
              willChange: 'scroll-position'
            }
          },
          this.props.children
        )
      );
    }
  }]);
  return WindowedScroller;
}(React.Component);
WindowedScroller.defaultProps = {
  onViewportChange: function onViewportChange() {}
};
WindowedScroller.propTypes = {
  onViewportChange: PropTypes.func,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)]).isRequired
};
WindowedScroller.childContextTypes = {
  virtualHost: PropTypes.object.isRequired
};

var stickyProp = void 0;
var testCSSProp = function testCSSProp(property, value, noPrefixes) {
  var prop = property + ':';
  // eslint-disable-next-line no-undef
  var el = document.createElement('test');
  var mStyle = el.style;

  if (!noPrefixes) {
    mStyle.cssText = prop + ['-webkit-', '-moz-', '-ms-', '-o-', ''].join(value + ';' + prop) + value + ';';
  } else {
    mStyle.cssText = prop + value;
  }
  return mStyle[property];
};

function getVisibleItems(options) {
  var viewportStart = options.viewport.start;
  var viewportSize = options.viewport.size;

  var visibleItemMetas = [];
  var stickyItemsMetas = [];

  var index = 0;
  var offset = 0;
  while (index < options.itemCount) {
    var itemInfo = options.itemInfo(index);
    var itemSize = itemInfo.size;
    var itemStick = itemInfo.stick;

    if (itemStick === 'before' && offset <= viewportStart) {
      stickyItemsMetas.push({
        index: index, offset: offset, size: itemSize, stick: itemStick
      });
      viewportStart += itemSize;
    }

    if ((offset + itemSize > viewportStart && offset + itemSize < viewportStart + viewportSize || offset < viewportStart + viewportSize && offset > viewportStart || offset <= viewportStart && offset + itemSize >= viewportStart + viewportSize) && itemSize > 0 && itemStick === false) {
      visibleItemMetas.push({
        index: index, offset: offset, size: itemSize, stick: false
      });
    }

    index += 1;
    offset += itemSize;
  }

  return {
    visibleItemMetas: [].concat(stickyItemsMetas, visibleItemMetas),
    fullSize: offset,
    stickyBeforeSize: stickyItemsMetas.reduce(function (accumulator, meta) {
      return accumulator + meta.size;
    }, 0)
  };
}

var VirtualBox = function (_React$Component) {
  inherits(VirtualBox, _React$Component);

  function VirtualBox() {
    classCallCheck(this, VirtualBox);
    return possibleConstructorReturn(this, (VirtualBox.__proto__ || Object.getPrototypeOf(VirtualBox)).apply(this, arguments));
  }

  createClass(VirtualBox, [{
    key: 'getChildContext',
    value: function getChildContext() {
      var _viewport;

      var _props = this.props,
          direction = _props.direction,
          position = _props.position,
          stick = _props.stick;
      var viewport = this.context.virtualHost.viewport;


      var positionProp = direction === 'horizontal' ? 'left' : 'top';
      var crossPositionProp = direction === 'horizontal' ? 'top' : 'left';
      var sizeProp = direction === 'horizontal' ? 'width' : 'height';
      var crossSizeProp = direction === 'horizontal' ? 'height' : 'width';

      return {
        virtualHost: {
          viewport: (_viewport = {}, defineProperty(_viewport, positionProp, stick ? position : Math.max(viewport[positionProp] - position, 0)), defineProperty(_viewport, crossPositionProp, viewport[crossPositionProp]), defineProperty(_viewport, sizeProp, stick ? viewport[sizeProp] : Math.min(viewport[positionProp] + viewport[sizeProp] - position, viewport[sizeProp])), defineProperty(_viewport, crossSizeProp, viewport[crossSizeProp]), _viewport)
        }
      };
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      if (stickyProp === undefined) {
        stickyProp = testCSSProp('position', 'sticky');
      }
    }
  }, {
    key: 'getItemStyles',
    value: function getItemStyles(_ref) {
      var _babelHelpers$extends2;

      var position = _ref.position,
          size = _ref.size,
          stick = _ref.stick;
      var _props2 = this.props,
          direction = _props2.direction,
          crossSize = _props2.crossSize;
      var viewport = this.context.virtualHost.viewport;


      var positionProp = direction === 'horizontal' ? 'left' : 'top';
      var sizeProp = direction === 'horizontal' ? 'width' : 'height';
      var crossSizeProp = direction === 'vertical' ? 'width' : 'height';

      var styles = defineProperty({
        position: 'absolute'
      }, positionProp, position + 'px');

      if (stick) {
        if (stickyProp) {
          styles = _extends({}, styles, defineProperty({

            zIndex: 1,
            position: stickyProp
          }, positionProp, position + (viewport[positionProp + 'Stick'] || 0) + 'px'));
        } else if (direction === 'vertical') {
          var _styles2;

          styles = (_styles2 = {
            zIndex: 1,
            position: 'fixed'
          }, defineProperty(_styles2, crossSizeProp, viewport[crossSizeProp] + 'px'), defineProperty(_styles2, 'overflow', 'hidden'), defineProperty(_styles2, 'WebkitBackfaceVisibility', 'hidden'), defineProperty(_styles2, 'backfaceVisibility', 'hidden'), _styles2);
          /* children = (
              <div
                style={{
                  marginLeft: -viewport[crossPositionProp] + 'px',
                  width: '100%',
                  height: '100%'
                }}
              >
                  {children}
              </div>
          ); */
        }
      }

      return _extends({}, styles, (_babelHelpers$extends2 = {
        display: 'block'
      }, defineProperty(_babelHelpers$extends2, sizeProp, size + 'px'), defineProperty(_babelHelpers$extends2, crossSizeProp, crossSize + 'px'), _babelHelpers$extends2));
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this,
          _babelHelpers$extends3;

      var _props3 = this.props,
          direction = _props3.direction,
          position = _props3.position,
          stick = _props3.stick,
          iref = _props3.iref,
          rootTagTemplate = _props3.rootTagTemplate;
      var viewport = this.context.virtualHost.viewport;


      var positionProp = direction === 'horizontal' ? 'left' : 'top';
      var sizeProp = direction === 'horizontal' ? 'width' : 'height';

      var _getVisibleItems = getVisibleItems({
        viewport: { start: stick ? position : viewport[positionProp], size: viewport[sizeProp] },
        itemCount: this.props.itemCount,
        itemInfo: this.props.itemInfo
      }),
          visibleItemMetas = _getVisibleItems.visibleItemMetas,
          fullSize = _getVisibleItems.fullSize;

      var visibleItems = visibleItemMetas.map(function (visibleItemMeta) {
        var styles = _this2.getItemStyles({
          position: visibleItemMeta.offset,
          size: visibleItemMeta.size,
          stick: visibleItemMeta.stick
        });

        return React.cloneElement(_this2.props.itemTemplate(visibleItemMeta.index, visibleItemMeta.offset, styles), {
          key: '' + visibleItemMeta.index,
          style: styles
        });
      });

      var RootTag = this.props.rootTag;
      var rootTagProps = {
        style: _extends({
          position: 'relative'
        }, this.props.style, (_babelHelpers$extends3 = {}, defineProperty(_babelHelpers$extends3, sizeProp, fullSize + 'px'), defineProperty(_babelHelpers$extends3, 'display', 'block'), _babelHelpers$extends3)),
        className: this.props.className
      };

      if (rootTagTemplate) {
        return rootTagTemplate(_extends({}, rootTagProps, {
          children: visibleItems
        }));
      }

      return React.createElement(
        RootTag,
        _extends({}, iref ? { ref: iref } : {}, rootTagProps),
        visibleItems
      );
    }
  }]);
  return VirtualBox;
}(React.Component);
VirtualBox.defaultProps = {
  rootTag: 'div',
  rootTagTemplate: undefined,
  className: '',
  style: {},
  position: 0,
  stick: false,
  crossSize: 0,
  iref: undefined
};
VirtualBox.propTypes = {
  rootTag: PropTypes.string,
  rootTagTemplate: PropTypes.func,
  className: PropTypes.string,
  style: PropTypes.object,
  position: PropTypes.number,
  stick: PropTypes.bool,
  iref: PropTypes.func,
  crossSize: PropTypes.number,
  direction: PropTypes.oneOf(['vertical', 'horizontal']).isRequired,
  itemCount: PropTypes.number.isRequired,
  itemInfo: PropTypes.func.isRequired,
  itemTemplate: PropTypes.func.isRequired
};
VirtualBox.contextTypes = {
  virtualHost: PropTypes.object
};
VirtualBox.childContextTypes = {
  virtualHost: PropTypes.object.isRequired
};

var DEFAULT_HEIGHT = 37;
var MINIMAL_COLUMN_WIDTH$1 = 120;

var calculateColumnWidths = function calculateColumnWidths(columns, tableWidth) {
  var occupiedWidth = columns.map(function (column) {
    return column.width;
  }).filter(function (width) {
    return !!width;
  }).reduce(function (accum, width) {
    return accum + width;
  }, 0);
  var restWidthSpread = columns.map(function (column) {
    return column.width;
  }).filter(function (width) {
    return !width;
  }).length;
  var calculatedWidth = (tableWidth - occupiedWidth) / restWidthSpread;
  var widthForAutoColumn = Math.max(calculatedWidth, MINIMAL_COLUMN_WIDTH$1);

  return columns.map(function (column) {
    return column.width;
  }).map(function (width) {
    return !!width && width !== 'auto' ? width : widthForAutoColumn;
  });
};

var VirtualTable = function (_React$Component) {
  inherits(VirtualTable, _React$Component);

  function VirtualTable(props) {
    classCallCheck(this, VirtualTable);

    var _this = possibleConstructorReturn(this, (VirtualTable.__proto__ || Object.getPrototypeOf(VirtualTable)).call(this, props));

    _this.state = {
      viewportWidth: 0
    };

    _this.rowHeight = function (row) {
      var height = row.height || DEFAULT_HEIGHT;
      if (height === 'auto') {
        height = DEFAULT_HEIGHT;
      }
      return height;
    };
    return _this;
  }

  createClass(VirtualTable, [{
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          headerRows = _props.headerRows,
          bodyRows = _props.bodyRows,
          columns = _props.columns,
          cellTemplate = _props.cellTemplate,
          rowTemplate = _props.rowTemplate;


      var columnWidths = calculateColumnWidths(columns, this.state.viewportWidth);
      var scrollWidth = columnWidths.reduce(function (accum, width) {
        return accum + width;
      }, 0);

      var tableRow = function tableRow(row, position) {
        var columnsWithColSpan = getTableRowColumnsWithColSpan(columns, row.colSpanStart);
        return React.createElement(VirtualBox, {
          rootTag: 'tr',
          rootTagTemplate: function rootTagTemplate(params) {
            return rowTemplate(_extends({ tableRow: row }, params));
          },
          position: position,
          crossSize: _this2.rowHeight(row),
          direction: 'horizontal',
          itemCount: columnsWithColSpan.length,
          itemInfo: function itemInfo(columnIndex) {
            var columnWithColSpan = columnsWithColSpan[columnIndex];
            var size = !columnWithColSpan.colSpan ? columnWidths[columnIndex] : columns.slice(columnIndex).reduce(function (accum, column) {
              return accum + columnWidths[columns.indexOf(column)];
            }, 0);

            return {
              size: size,
              stick: false
            };
          },
          itemTemplate: function itemTemplate(columnIndex, _, style) {
            return cellTemplate({ tableRow: row, tableColumn: columnsWithColSpan[columnIndex], style: style });
          }
        });
      };

      var tableHead = function tableHead(position) {
        return React.createElement(VirtualBox, {
          key: 'thead',
          rootTag: 'thead',

          iref: function iref(ref) {
            if (!ref) return;
            var style = ref.style;
            // eslint-disable-next-line no-undef

            style.backgroundColor = window.getComputedStyle(document.body).backgroundColor;
          },

          position: position,
          stick: true,
          direction: 'vertical',
          itemCount: headerRows.length,
          itemInfo: function itemInfo(rowIndex) {
            return {
              size: _this2.rowHeight(headerRows[rowIndex]),
              stick: false
            };
          },
          itemTemplate: function itemTemplate(rowIndex, rowPosition) {
            return tableRow(headerRows[rowIndex], rowPosition);
          }
        });
      };

      var tableBody = function tableBody(position) {
        return React.createElement(VirtualBox, {
          key: 'tbody',
          rootTag: 'tbody',

          position: position,
          direction: 'vertical',
          itemCount: bodyRows.length,
          itemInfo: function itemInfo(rowIndex) {
            return {
              size: _this2.rowHeight(bodyRows[rowIndex]),
              stick: false
            };
          },
          itemTemplate: function itemTemplate(rowIndex, rowPosition) {
            return tableRow(bodyRows[rowIndex], rowPosition);
          }
        });
      };

      return React.createElement(
        'div',
        {
          style: { height: '530px' }
        },
        React.createElement(
          WindowedScroller,
          {
            onViewportChange: function onViewportChange(viewport) {
              return _this2.setState({ viewportWidth: viewport.width });
            }
          },
          React.createElement(VirtualBox, {
            rootTag: 'table',
            className: 'table',
            style: { zIndex: 0, margin: 0 },

            crossSize: scrollWidth,
            direction: 'vertical',
            itemCount: 2,
            itemInfo: function itemInfo(rowIndex) {
              var size = rowIndex === 0 ? headerRows.reduce(function (accum, row) {
                return accum + _this2.rowHeight(row);
              }, 0) : bodyRows.reduce(function (accum, row) {
                return accum + _this2.rowHeight(row);
              }, 0);

              return {
                size: size,
                stick: rowIndex === 0 ? 'before' : false
              };
            },
            itemTemplate: function itemTemplate(rowIndex, position) {
              return rowIndex === 0 ? tableHead(position) : tableBody(position);
            }
          })
        )
      );
    }
  }]);
  return VirtualTable;
}(React.Component);

VirtualTable.propTypes = {
  headerRows: PropTypes.array.isRequired,
  bodyRows: PropTypes.array.isRequired,
  columns: PropTypes.array.isRequired,
  cellTemplate: PropTypes.func.isRequired,
  rowTemplate: PropTypes.func.isRequired
};

var tableLayoutTemplate$1 = function tableLayoutTemplate(props) {
  return React.createElement(VirtualTable, props);
};
var defaultRowTemplate$1 = function defaultRowTemplate(props) {
  return React.createElement(TableRow, props);
};
var defaultNoDataRowTemplate$1 = function defaultNoDataRowTemplate(props) {
  return React.createElement(TableRow, props);
};
var defaultCellTemplate$2 = function defaultCellTemplate(props) {
  return React.createElement(TableCell, props);
};
var defaultStubCellTemplate$1 = function defaultStubCellTemplate(props) {
  return React.createElement(TableStubCell, props);
};
var defaultStubHeaderCellTemplate$1 = function defaultStubHeaderCellTemplate(props) {
  return React.createElement(TableStubHeaderCell, props);
};
var defaultNoDataCellTemplate$1 = function defaultNoDataCellTemplate(props) {
  return React.createElement(TableNoDataCell, props);
};

var VirtualTableView = function (_React$PureComponent) {
  inherits(VirtualTableView, _React$PureComponent);

  function VirtualTableView() {
    classCallCheck(this, VirtualTableView);
    return possibleConstructorReturn(this, (VirtualTableView.__proto__ || Object.getPrototypeOf(VirtualTableView)).apply(this, arguments));
  }

  createClass(VirtualTableView, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          tableCellTemplate = _props.tableCellTemplate,
          tableRowTemplate = _props.tableRowTemplate,
          tableNoDataRowTemplate = _props.tableNoDataRowTemplate,
          tableStubCellTemplate = _props.tableStubCellTemplate,
          tableStubHeaderCellTemplate = _props.tableStubHeaderCellTemplate,
          tableNoDataCellTemplate = _props.tableNoDataCellTemplate,
          restProps = objectWithoutProperties(_props, ['tableCellTemplate', 'tableRowTemplate', 'tableNoDataRowTemplate', 'tableStubCellTemplate', 'tableStubHeaderCellTemplate', 'tableNoDataCellTemplate']);


      return React.createElement(TableView, _extends({
        tableLayoutTemplate: tableLayoutTemplate$1,
        tableRowTemplate: combineTemplates(tableRowTemplate, defaultRowTemplate$1),
        tableNoDataRowTemplate: combineTemplates(tableNoDataRowTemplate, defaultNoDataRowTemplate$1),
        tableCellTemplate: combineTemplates(tableCellTemplate, defaultCellTemplate$2),
        tableStubCellTemplate: combineTemplates(tableStubCellTemplate, defaultStubCellTemplate$1),
        tableStubHeaderCellTemplate: combineTemplates(tableStubHeaderCellTemplate, defaultStubHeaderCellTemplate$1),
        tableNoDataCellTemplate: combineTemplates(tableNoDataCellTemplate, defaultNoDataCellTemplate$1)
      }, restProps));
    }
  }]);
  return VirtualTableView;
}(React.PureComponent);

VirtualTableView.propTypes = {
  tableCellTemplate: PropTypes.func,
  tableRowTemplate: PropTypes.func,
  tableNoDataRowTemplate: PropTypes.func,
  tableStubCellTemplate: PropTypes.func,
  tableStubHeaderCellTemplate: PropTypes.func,
  tableNoDataCellTemplate: PropTypes.func
};

VirtualTableView.defaultProps = {
  tableCellTemplate: undefined,
  tableRowTemplate: undefined,
  tableNoDataRowTemplate: undefined,
  tableStubCellTemplate: undefined,
  tableStubHeaderCellTemplate: undefined,
  tableNoDataCellTemplate: undefined
};

var TableFilterCell = function TableFilterCell(_ref) {
  var style = _ref.style,
      filter = _ref.filter,
      setFilter = _ref.setFilter,
      children = _ref.children;
  return React.createElement(
    'th',
    {
      style: _extends({
        fontWeight: 'normal'
      }, style)
    },
    children || React.createElement('input', {
      type: 'text',
      className: 'form-control',
      value: filter ? filter.value : '',
      onChange: function onChange(e) {
        return setFilter(e.target.value ? { value: e.target.value } : null);
      }
    })
  );
};

TableFilterCell.propTypes = {
  style: PropTypes.object,
  filter: PropTypes.object,
  setFilter: PropTypes.func,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)])
};

TableFilterCell.defaultProps = {
  style: null,
  filter: null,
  setFilter: function setFilter() {},
  children: undefined
};

var defaultFilterCellTemplate = function defaultFilterCellTemplate(props) {
  return React.createElement(TableFilterCell, props);
};
var defaultFilterRowTemplate = function defaultFilterRowTemplate(props) {
  return React.createElement(TableRow, props);
};

var TableFilterRow$1 = function (_React$PureComponent) {
  inherits(TableFilterRow$$1, _React$PureComponent);

  function TableFilterRow$$1() {
    classCallCheck(this, TableFilterRow$$1);
    return possibleConstructorReturn(this, (TableFilterRow$$1.__proto__ || Object.getPrototypeOf(TableFilterRow$$1)).apply(this, arguments));
  }

  createClass(TableFilterRow$$1, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          filterCellTemplate = _props.filterCellTemplate,
          filterRowTemplate = _props.filterRowTemplate,
          restProps = objectWithoutProperties(_props, ['filterCellTemplate', 'filterRowTemplate']);


      return React.createElement(TableFilterRow, _extends({
        filterCellTemplate: combineTemplates(filterCellTemplate, defaultFilterCellTemplate),
        filterRowTemplate: combineTemplates(filterRowTemplate, defaultFilterRowTemplate)
      }, restProps));
    }
  }]);
  return TableFilterRow$$1;
}(React.PureComponent);

TableFilterRow$1.propTypes = {
  filterCellTemplate: PropTypes.func,
  filterRowTemplate: PropTypes.func
};

TableFilterRow$1.defaultProps = {
  filterCellTemplate: undefined,
  filterRowTemplate: undefined
};

var ResizingControlLine = function ResizingControlLine(_ref) {
  var resizing = _ref.resizing,
      style = _ref.style;

  var resizingControlLineBody = resizing && React.createElement('div', {
    className: 'bg-primary',
    style: {
      position: 'absolute',
      top: 0,
      left: 0,
      height: '100%',
      width: '100%'
    }
  });

  return React.createElement(
    'div',
    {
      style: _extends({
        position: 'absolute',
        backgroundColor: '#ddd',
        height: '50%',
        width: '1px',
        top: '25%'
      }, style)
    },
    resizingControlLineBody
  );
};

ResizingControlLine.propTypes = {
  resizing: PropTypes.bool.isRequired,
  style: PropTypes.object.isRequired
};

var ResizingControl = function (_React$PureComponent) {
  inherits(ResizingControl, _React$PureComponent);

  function ResizingControl(props) {
    classCallCheck(this, ResizingControl);

    var _this = possibleConstructorReturn(this, (ResizingControl.__proto__ || Object.getPrototypeOf(ResizingControl)).call(this, props));

    _this.state = {
      resizing: false
    };

    _this.onResizeStart = function (_ref2) {
      var x = _ref2.x;

      _this.resizeStartingX = x;
      _this.setState({ resizing: true });
    };
    _this.onResizeUpdate = function (_ref3) {
      var x = _ref3.x;
      var changeDraftColumnWidth = _this.props.changeDraftColumnWidth;

      changeDraftColumnWidth({ shift: x - _this.resizeStartingX });
    };
    _this.onResizeEnd = function (_ref4) {
      var x = _ref4.x;
      var changeColumnWidth = _this.props.changeColumnWidth;

      changeColumnWidth({ shift: x - _this.resizeStartingX });
      _this.setState({ resizing: false });
    };
    return _this;
  }

  createClass(ResizingControl, [{
    key: 'render',
    value: function render() {
      var resizing = this.state.resizing;


      return React.createElement(
        Draggable,
        {
          onStart: this.onResizeStart,
          onUpdate: this.onResizeUpdate,
          onEnd: this.onResizeEnd
        },
        React.createElement(
          'div',
          {
            style: {
              position: 'absolute',
              userSelect: 'none',
              MozUserSelect: 'none',
              WebkitUserSelect: 'none',
              top: 0,
              right: '-8px',
              width: '16px',
              height: '100%',
              cursor: 'col-resize',
              zIndex: 100
            }
          },
          React.createElement(ResizingControlLine, { resizing: resizing, style: { left: '5px' } }),
          React.createElement(ResizingControlLine, { resizing: resizing, style: { left: '7px' } })
        )
      );
    }
  }]);
  return ResizingControl;
}(React.PureComponent);

ResizingControl.propTypes = {
  changeColumnWidth: PropTypes.func.isRequired,
  changeDraftColumnWidth: PropTypes.func.isRequired
};

var GroupingControl = function GroupingControl(_ref) {
  var align = _ref.align,
      groupByColumn = _ref.groupByColumn;

  var invertedAlign = align === 'left' ? 'right' : 'left';

  return React.createElement(
    'div',
    {
      onClick: function onClick(e) {
        e.stopPropagation();
        groupByColumn();
      },
      style: {
        float: invertedAlign,
        textAlign: invertedAlign,
        width: '14px'
      }
    },
    React.createElement('i', {
      className: 'glyphicon glyphicon-th-list',
      style: {
        top: '0',
        fontSize: '9px',
        margin: '-5px',
        padding: '5px',
        cursor: 'pointer'
      }
    })
  );
};

GroupingControl.propTypes = {
  align: PropTypes.string.isRequired,
  groupByColumn: PropTypes.func.isRequired
};

var SortingControl = function SortingControl(_ref) {
  var align = _ref.align,
      sortingDirection = _ref.sortingDirection,
      columnTitle = _ref.columnTitle;
  return align === 'right' ? React.createElement(
    'span',
    {
      className: sortingDirection ? 'text-primary' : ''
    },
    React.createElement(SortingIndicator, {
      direction: sortingDirection,
      style: { visibility: sortingDirection ? 'visible' : 'hidden' }
    }),
    '\xA0',
    columnTitle
  ) : React.createElement(
    'span',
    {
      className: sortingDirection ? 'text-primary' : ''
    },
    columnTitle,
    '\xA0',
    React.createElement(SortingIndicator, {
      direction: sortingDirection,
      style: { visibility: sortingDirection ? 'visible' : 'hidden' }
    })
  );
};

SortingControl.propTypes = {
  align: PropTypes.string.isRequired,
  sortingDirection: PropTypes.oneOf(['asc', 'desc']),
  columnTitle: PropTypes.string.isRequired
};

SortingControl.defaultProps = {
  sortingDirection: null
};

var TableHeaderCell = function (_React$PureComponent) {
  inherits(TableHeaderCell, _React$PureComponent);

  function TableHeaderCell(props) {
    classCallCheck(this, TableHeaderCell);

    var _this = possibleConstructorReturn(this, (TableHeaderCell.__proto__ || Object.getPrototypeOf(TableHeaderCell)).call(this, props));

    _this.state = {
      dragging: false
    };

    _this.onCellClick = function (e) {
      var _this$props = _this.props,
          allowSorting = _this$props.allowSorting,
          changeSortingDirection = _this$props.changeSortingDirection;

      if (!allowSorting) return;
      e.stopPropagation();
      var cancelSortingRelatedKey = e.metaKey || e.ctrlKey;
      changeSortingDirection({
        keepOther: e.shiftKey || cancelSortingRelatedKey,
        cancel: cancelSortingRelatedKey
      });
    };
    return _this;
  }

  createClass(TableHeaderCell, [{
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          style = _props.style,
          column = _props.column,
          tableColumn = _props.tableColumn,
          allowSorting = _props.allowSorting,
          sortingDirection = _props.sortingDirection,
          allowGroupingByClick = _props.allowGroupingByClick,
          groupByColumn = _props.groupByColumn,
          allowDragging = _props.allowDragging,
          dragPayload = _props.dragPayload,
          allowResizing = _props.allowResizing,
          changeColumnWidth = _props.changeColumnWidth,
          changeDraftColumnWidth = _props.changeDraftColumnWidth;
      var dragging = this.state.dragging;

      var align = column.align || 'left';
      var columnTitle = column.title || column.name;

      var cellLayout = React.createElement(
        'th',
        {
          style: _extends({
            position: 'relative'
          }, allowSorting || allowDragging ? {
            userSelect: 'none',
            MozUserSelect: 'none',
            WebkitUserSelect: 'none'
          } : {}, allowSorting || allowDragging ? { cursor: 'pointer' } : null, dragging || tableColumn.draft ? { opacity: 0.3 } : null, style),
          onClick: this.onCellClick
        },
        allowGroupingByClick && React.createElement(GroupingControl, {
          align: align,
          groupByColumn: groupByColumn
        }),
        React.createElement(
          'div',
          {
            style: _extends({}, allowGroupingByClick ? defineProperty({}, 'margin' + (column.align === 'right' ? 'Left' : 'Right'), '14px') : null, {
              textAlign: align,
              whiteSpace: 'nowrap',
              overflow: 'hidden',
              textOverflow: 'ellipsis'
            })
          },
          allowSorting ? React.createElement(SortingControl, {
            align: align,
            sortingDirection: sortingDirection,
            columnTitle: columnTitle
          }) : columnTitle
        ),
        allowResizing && React.createElement(ResizingControl, {
          changeColumnWidth: changeColumnWidth,
          changeDraftColumnWidth: changeDraftColumnWidth
        })
      );

      return allowDragging ? React.createElement(
        DragSource,
        {
          ref: function ref(element) {
            _this2.cellRef = element;
          },
          getPayload: function getPayload() {
            return dragPayload;
          },
          onStart: function onStart() {
            return _this2.setState({ dragging: true });
          },
          onEnd: function onEnd() {
            return _this2.cellRef && _this2.setState({ dragging: false });
          }
        },
        cellLayout
      ) : cellLayout;
    }
  }]);
  return TableHeaderCell;
}(React.PureComponent);

TableHeaderCell.propTypes = {
  tableColumn: PropTypes.object,
  column: PropTypes.shape({
    title: PropTypes.string
  }).isRequired,
  style: PropTypes.object,
  allowSorting: PropTypes.bool,
  sortingDirection: PropTypes.oneOf(['asc', 'desc', null]),
  changeSortingDirection: PropTypes.func,
  allowGroupingByClick: PropTypes.bool,
  groupByColumn: PropTypes.func,
  allowDragging: PropTypes.bool,
  dragPayload: PropTypes.any,
  allowResizing: PropTypes.bool,
  changeColumnWidth: PropTypes.func,
  changeDraftColumnWidth: PropTypes.func
};

TableHeaderCell.defaultProps = {
  tableColumn: {},
  style: null,
  allowSorting: false,
  sortingDirection: undefined,
  changeSortingDirection: undefined,
  allowGroupingByClick: false,
  groupByColumn: undefined,
  allowDragging: false,
  dragPayload: null,
  allowResizing: false,
  changeColumnWidth: undefined,
  changeDraftColumnWidth: undefined
};

var defaultHeaderCellTemplate = function defaultHeaderCellTemplate(props) {
  return React.createElement(TableHeaderCell, props);
};
var defaultHeaderRowTemplate = function defaultHeaderRowTemplate(props) {
  return React.createElement(TableRow, props);
};

var TableHeaderRow$1 = function (_React$PureComponent) {
  inherits(TableHeaderRow$$1, _React$PureComponent);

  function TableHeaderRow$$1() {
    classCallCheck(this, TableHeaderRow$$1);
    return possibleConstructorReturn(this, (TableHeaderRow$$1.__proto__ || Object.getPrototypeOf(TableHeaderRow$$1)).apply(this, arguments));
  }

  createClass(TableHeaderRow$$1, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          headerCellTemplate = _props.headerCellTemplate,
          headerRowTemplate = _props.headerRowTemplate,
          restProps = objectWithoutProperties(_props, ['headerCellTemplate', 'headerRowTemplate']);


      return React.createElement(TableHeaderRow, _extends({
        headerCellTemplate: combineTemplates(headerCellTemplate, defaultHeaderCellTemplate),
        headerRowTemplate: combineTemplates(headerRowTemplate, defaultHeaderRowTemplate)
      }, restProps));
    }
  }]);
  return TableHeaderRow$$1;
}(React.PureComponent);

TableHeaderRow$1.propTypes = {
  headerCellTemplate: PropTypes.func,
  headerRowTemplate: PropTypes.func
};

TableHeaderRow$1.defaultProps = {
  headerCellTemplate: undefined,
  headerRowTemplate: undefined
};

var EditCell = function EditCell(_ref) {
  var column = _ref.column,
      value = _ref.value,
      onValueChange = _ref.onValueChange,
      style = _ref.style,
      children = _ref.children;
  return React.createElement(
    'td',
    {
      style: _extends({
        verticalAlign: 'middle',
        padding: '1px'
      }, style)
    },
    children || React.createElement('input', {
      type: 'text',
      className: 'form-control',
      value: value,
      onChange: function onChange(e) {
        return onValueChange(e.target.value);
      },
      style: { width: '100%', textAlign: column.align }
    })
  );
};
EditCell.propTypes = {
  column: PropTypes.object,
  value: PropTypes.any,
  onValueChange: PropTypes.func.isRequired,
  style: PropTypes.object,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)])
};
EditCell.defaultProps = {
  column: {},
  value: '',
  style: {},
  children: undefined
};

var defaultEditCellTemplate = function defaultEditCellTemplate(props) {
  return React.createElement(EditCell, props);
};
var defaultEditRowTemplate = function defaultEditRowTemplate(props) {
  return React.createElement(TableRow, props);
};

var TableEditRow$1 = function (_React$PureComponent) {
  inherits(TableEditRow$$1, _React$PureComponent);

  function TableEditRow$$1() {
    classCallCheck(this, TableEditRow$$1);
    return possibleConstructorReturn(this, (TableEditRow$$1.__proto__ || Object.getPrototypeOf(TableEditRow$$1)).apply(this, arguments));
  }

  createClass(TableEditRow$$1, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          editCellTemplate = _props.editCellTemplate,
          editRowTemplate = _props.editRowTemplate,
          restProps = objectWithoutProperties(_props, ['editCellTemplate', 'editRowTemplate']);


      return React.createElement(TableEditRow, _extends({
        editCellTemplate: combineTemplates(editCellTemplate, defaultEditCellTemplate),
        editRowTemplate: combineTemplates(editRowTemplate, defaultEditRowTemplate)
      }, restProps));
    }
  }]);
  return TableEditRow$$1;
}(React.PureComponent);

TableEditRow$1.propTypes = {
  editCellTemplate: PropTypes.func,
  editRowTemplate: PropTypes.func
};

TableEditRow$1.defaultProps = {
  editCellTemplate: undefined,
  editRowTemplate: undefined
};

var CommandButton = function CommandButton(_ref) {
  var executeCommand = _ref.executeCommand,
      text = _ref.text;
  return React.createElement(
    'button',
    {
      className: 'btn btn-link',
      onClick: function onClick(e) {
        executeCommand();
        e.stopPropagation();
      }
    },
    text
  );
};

CommandButton.propTypes = {
  executeCommand: PropTypes.func.isRequired,
  text: PropTypes.string.isRequired
};

var EditCommandHeadingCell = function EditCommandHeadingCell(_ref2) {
  var addRow = _ref2.addRow,
      commandTemplate = _ref2.commandTemplate,
      allowAdding = _ref2.allowAdding,
      style = _ref2.style,
      addCommandText = _ref2.addCommandText;
  return React.createElement(
    'th',
    {
      style: _extends({
        whiteSpace: 'nowrap',
        textAlign: 'center',
        padding: 0
      }, style)
    },
    allowAdding && commandTemplate({
      id: 'add',
      executeCommand: addRow,
      text: addCommandText
    })
  );
};

EditCommandHeadingCell.propTypes = {
  addRow: PropTypes.func.isRequired,
  commandTemplate: PropTypes.func.isRequired,
  addCommandText: PropTypes.string,
  allowAdding: PropTypes.bool.isRequired,
  style: PropTypes.object
};

EditCommandHeadingCell.defaultProps = {
  style: {},
  addCommandText: 'New'
};

var EditCommandCell = function EditCommandCell(_ref3) {
  var startEditing = _ref3.startEditing,
      deleteRow = _ref3.deleteRow,
      cancelEditing = _ref3.cancelEditing,
      commitChanges = _ref3.commitChanges,
      isEditing = _ref3.isEditing,
      commandTemplate = _ref3.commandTemplate,
      allowEditing = _ref3.allowEditing,
      allowDeleting = _ref3.allowDeleting,
      style = _ref3.style,
      editCommandText = _ref3.editCommandText,
      deleteCommandText = _ref3.deleteCommandText,
      commitCommandText = _ref3.commitCommandText,
      cancelCommandText = _ref3.cancelCommandText;

  var commands = [];
  if (!isEditing) {
    if (allowEditing) {
      commands.push({
        id: 'edit',
        executeCommand: startEditing,
        text: editCommandText
      });
    }
    if (allowDeleting) {
      commands.push({
        id: 'delete',
        executeCommand: deleteRow,
        text: deleteCommandText
      });
    }
  } else {
    commands = [{
      id: 'commit',
      executeCommand: commitChanges,
      text: commitCommandText
    }, {
      id: 'cancel',
      executeCommand: cancelEditing,
      text: cancelCommandText
    }];
  }
  return React.createElement(
    'td',
    {
      style: _extends({
        whiteSpace: 'nowrap',
        textAlign: 'center',
        padding: 0
      }, style)
    },
    commands.map(function (command) {
      return React.createElement(
        'span',
        { key: command.id },
        commandTemplate(command)
      );
    })
  );
};

EditCommandCell.propTypes = {
  startEditing: PropTypes.func.isRequired,
  deleteRow: PropTypes.func.isRequired,
  cancelEditing: PropTypes.func.isRequired,
  commitChanges: PropTypes.func.isRequired,
  isEditing: PropTypes.bool.isRequired,
  allowEditing: PropTypes.bool.isRequired,
  allowDeleting: PropTypes.bool.isRequired,
  commandTemplate: PropTypes.func.isRequired,
  editCommandText: PropTypes.string,
  deleteCommandText: PropTypes.string,
  commitCommandText: PropTypes.string,
  cancelCommandText: PropTypes.string,
  style: PropTypes.object
};

EditCommandCell.defaultProps = {
  style: {},
  editCommandText: 'Edit',
  deleteCommandText: 'Delete',
  commitCommandText: 'Save',
  cancelCommandText: 'Cancel'
};

var defaultCellTemplate$3 = function defaultCellTemplate(props) {
  return React.createElement(EditCommandCell, props);
};
var defaultHeadingCellTemplate = function defaultHeadingCellTemplate(props) {
  return React.createElement(EditCommandHeadingCell, props);
};
var defaultCommandTemplate = function defaultCommandTemplate(props) {
  return React.createElement(CommandButton, props);
};

var TableEditColumn$1 = function (_React$PureComponent) {
  inherits(TableEditColumn$$1, _React$PureComponent);

  function TableEditColumn$$1() {
    classCallCheck(this, TableEditColumn$$1);
    return possibleConstructorReturn(this, (TableEditColumn$$1.__proto__ || Object.getPrototypeOf(TableEditColumn$$1)).apply(this, arguments));
  }

  createClass(TableEditColumn$$1, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          cellTemplate = _props.cellTemplate,
          headingCellTemplate = _props.headingCellTemplate,
          commandTemplate = _props.commandTemplate,
          restProps = objectWithoutProperties(_props, ['cellTemplate', 'headingCellTemplate', 'commandTemplate']);


      return React.createElement(TableEditColumn, _extends({
        cellTemplate: combineTemplates(cellTemplate, defaultCellTemplate$3),
        headingCellTemplate: combineTemplates(headingCellTemplate, defaultHeadingCellTemplate),
        commandTemplate: combineTemplates(commandTemplate, defaultCommandTemplate)
      }, restProps));
    }
  }]);
  return TableEditColumn$$1;
}(React.PureComponent);

TableEditColumn$1.propTypes = {
  cellTemplate: PropTypes.func,
  headingCellTemplate: PropTypes.func,
  commandTemplate: PropTypes.func
};

TableEditColumn$1.defaultProps = {
  cellTemplate: undefined,
  headingCellTemplate: undefined,
  commandTemplate: undefined
};

var EmptyMessage = function EmptyMessage(_ref) {
  var text = _ref.text;
  return React.createElement(
    'div',
    { className: 'panel-body' },
    text
  );
};

EmptyMessage.propTypes = {
  text: PropTypes.string.isRequired
};

var TableColumnVisibility$1 = function TableColumnVisibility$$1(_ref) {
  var emptyMessageText = _ref.emptyMessageText,
      restProps = objectWithoutProperties(_ref, ['emptyMessageText']);

  var emptyMessageTemplate = function emptyMessageTemplate() {
    return React.createElement(EmptyMessage, { text: emptyMessageText });
  };

  return React.createElement(TableColumnVisibility, _extends({
    emptyMessageTemplate: emptyMessageTemplate
  }, restProps));
};

TableColumnVisibility$1.propTypes = {
  emptyMessageText: PropTypes.string
};

TableColumnVisibility$1.defaultProps = {
  emptyMessageText: 'Nothing to show'
};

export { Grid$1 as Grid, ColumnChooser$1 as ColumnChooser, DragDropContext$1 as DragDropContext, PagingPanel$1 as PagingPanel, GroupingPanel$1 as GroupingPanel, TableRowDetail$1 as TableRowDetail, TableGroupRow$1 as TableGroupRow, TableSelection$1 as TableSelection, TableView$1 as TableView, VirtualTableView, TableFilterRow$1 as TableFilterRow, TableHeaderRow$1 as TableHeaderRow, TableEditRow$1 as TableEditRow, TableEditColumn$1 as TableEditColumn, TableColumnVisibility$1 as TableColumnVisibility };
//# sourceMappingURL=dx-react-grid-bootstrap3.es.js.map
